
#if LIBMIRISDR_PRESENT == 1
#include <errno.h>
#include <signal.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "globdef.h"
#include "uidef.h"
#include "sdrdef.h"
#include "conf.h"
#include "fft1def.h"
#include "screendef.h"
#include "rusage.h"
#include "thrdef.h"
#include "options.h"
#include "vernr.h"
#include "hwaredef.h"
#include "thrdef.h"

#if(OSNUM == OSNUM_WINDOWS)
#include <windows.h>
#endif

#if(OSNUM == OSNUM_LINUX)
#include <pthread.h>
#include <unistd.h>
#endif

#include <mirisdr.h>

#define MINIMAL_BUF_LENGTH		512
#define MAXIMAL_BUF_LENGTH		(256 * 16384)



P_MIRISDR mirics;

static mirisdr_dev_t *dev_mirics;

char *mirics_parm_text[MAX_MIRISDR_PARM]={"Sampling speed",
                                     "Serno1",
                                     "Serno2",
                                     "Serno3",
                                     "Freq adjust",
                                     "Check"};

char *mirics_parfil_name="par_mirics";

static  void mirisdr_callback(unsigned char *buf, uint32_t len, void *ctx)
{
(void) ctx;
unsigned int i;
short int *iz;
// assume len is in bytes.
for(i=0; i<len/2; i++)
  {
  iz=(short int*)&timf1_char[timf1p_sdr];
  iz[0]=buf[i];
  timf1p_sdr=(timf1p_sdr+2)&timf1_bytemask;
  }
if( ((timf1p_sdr-timf1p_pa+timf1_bytes)&timf1_bytemask) >= 
                                                     snd[RXAD].block_bytes)
  {
  lir_set_event(EVENT_HWARE1_RXREADY);
  }
}


void mirisdr_starter(void)
{
int k, no_of_buffers;

k=snd[RXAD].block_bytes/2;
if(k < MINIMAL_BUF_LENGTH)k=MINIMAL_BUF_LENGTH;
if(k > MAXIMAL_BUF_LENGTH)k=MAXIMAL_BUF_LENGTH;
// Allocate buffers for 50 ms minimum.
no_of_buffers=(int)((0.1*mirics.sampling_speed)/k);
// Never use less than 4 buffers.
if(no_of_buffers < 4)no_of_buffers=4;
while(thread_status_flag[THREAD_MIRISDR_INPUT]!=THRFLAG_ACTIVE)
  {
  lir_sleep(10000);
  if(kill_all_flag)return;
  }
mirisdr_set_center_freq(dev_mirics, 500000000);
mirisdr_read_async(dev_mirics, mirisdr_callback, NULL,
				      no_of_buffers, k);
}




void set_mirisdr_att(void)
{}
void set_mirisdr_frequency(void)
{}

void mirics_input(void)
{
#if RUSAGE_OLD == TRUE
int local_workload_counter;
#endif
int i, j;
int rxin_local_workload_reset;
char s[80];
double dt1, read_start_time, total_reads;
int timing_loop_counter,timing_loop_counter_max,initial_skip_flag;
int local_att_counter;
int local_nco_counter;
char vendor[256], product[256], serial[256];
int *ise;
unsigned int no_of_mirics;
uint32_t idx;
lir_init_event(EVENT_HWARE1_RXREADY);
#if OSNUM == OSNUM_LINUX
clear_thread_times(THREAD_MIRISDR_INPUT);
#endif
#if RUSAGE_OLD == TRUE
local_workload_counter=workload_counter;
#endif
local_att_counter=sdr_att_counter;
local_nco_counter=sdr_nco_counter;
dt1=current_time();
i=read_sdrpar(mirics_parfil_name, MAX_MIRISDR_PARM, 
                                     mirics_parm_text, (int*)((void*)&mirics));
if(i != 0 || mirics.check != MIRISDRPAR_VERNR)
  {
  lirerr(1377);
  goto mirics_init_error_exit;
  } 
if(ui.rx_ad_speed != (int)((rint)((mirics.sampling_speed*
                     (100000000L+(double)mirics.freq_adjust))/100000000L)))
  {
  lirerr(1377);
  goto mirics_init_error_exit;
  } 
timf1p_sdr=timf1p_pa;  
j=0;
while(sdr == -1)
  {
  ise=(int*)(void*)serial;
  dev_mirics=NULL;
  no_of_mirics = mirisdr_get_device_count();
  for(idx=0; idx<no_of_mirics; idx++)
    {
    mirisdr_get_device_usb_strings(idx, vendor, product, serial);
    if( mirics.sernum1 == ise[0] &&
        mirics.sernum2 == ise[1] &&
        mirics.sernum3 == ise[2])
      {  
      sdr=mirisdr_open(&dev_mirics, idx);
      if(kill_all_flag)return;
      if(sdr >= 0)
        {
        i=mirisdr_set_sample_rate(dev_mirics, (uint32_t)mirics.sampling_speed);
        if(i < 0)
          {
          lirerr(1368);
opnerr:;        
          mirisdr_close(dev_mirics);
          return;
          }
// Reset endpoint before we start reading from the MIRISDR (mandatory)
        i=mirisdr_reset_buffer(dev_mirics);
        if(i < 0)
          {
          lirerr(1378);
          goto opnerr;
          }
        linrad_thread_create(THREAD_MIRISDR_STARTER);
        break;
        }
      else
        {
        if(j==0)
          {
          clear_screen();
          j=1;
          }
        sprintf(s,"Waiting %.2f", current_time()-dt1);
        lir_text(3,5,s);
        lir_refresh_screen();
        if(kill_all_flag)goto mirics_init_error_exit;
        lir_sleep(100000);
        }
      }
    }
  }
set_hardware_rx_gain();
set_mirisdr_att();
set_mirisdr_frequency();
fft1_block_timing();
fft1_hz_per_point=(float)ui.rx_ad_speed/fft1_size;
if(thread_command_flag[THREAD_SCREEN]!=THRFLAG_NOT_ACTIVE)
  {
  while(thread_status_flag[THREAD_SCREEN]!=THRFLAG_ACTIVE &&
        thread_status_flag[THREAD_SCREEN]!=THRFLAG_IDLE &&
        thread_status_flag[THREAD_SCREEN]!=THRFLAG_SEM_WAIT)
    {
    if(thread_command_flag[THREAD_MIRISDR_INPUT] ==
                                           THRFLAG_KILL)goto mirics_error_exit1;
    lir_sleep(10000);
    }
  }
#include "timing_setup.c"
thread_status_flag[THREAD_MIRISDR_INPUT]=THRFLAG_ACTIVE;
while(!kill_all_flag && 
            thread_command_flag[THREAD_MIRISDR_INPUT] == THRFLAG_ACTIVE)
  {
#if RUSAGE_OLD == TRUE
  if(local_workload_counter != workload_counter)
    {
    local_workload_counter=workload_counter;
    make_thread_times(THREAD_MIRISDR_INPUT);
    }
#endif
  if(local_att_counter != sdr_att_counter)
    {
    local_att_counter=sdr_att_counter;
    set_mirisdr_att();
    }
  if(local_nco_counter != sdr_nco_counter)
    {
    local_nco_counter=sdr_nco_counter;
    set_mirisdr_frequency();
    }
  lir_await_event(EVENT_HWARE1_RXREADY);
  if(kill_all_flag)goto mirics_error_exit1;
  while (!kill_all_flag && ((timf1p_pa-timf1p_sdr+timf1_bytes)&
                                  timf1_bytemask)>snd[RXAD].block_bytes)
    {
#include "input_speed.c"
    finish_rx_read();
    }
  }
mirics_error_exit1:;
mirisdr_cancel_async(dev_mirics);
mirisdr_close(dev_mirics);
lir_join(THREAD_MIRISDR_STARTER);
mirics_init_error_exit:;
sdr=-1;
thread_status_flag[THREAD_MIRISDR_INPUT]=THRFLAG_RETURNED;
while(thread_command_flag[THREAD_MIRISDR_INPUT] != THRFLAG_NOT_ACTIVE)
  {
  lir_sleep(1000);
  }
lir_close_event(EVENT_HWARE1_RXREADY);
} 

int display_mirics_parm_info(int *line)
{
char s[80];
int errcod;
errcod=read_sdrpar(mirics_parfil_name, MAX_MIRISDR_PARM, 
                               mirics_parm_text, (int*)((void*)&mirics));
if(errcod == 0)
  {
  settextcolor(7);
  sprintf(s,"sampling rate = %i, Xtal adjust = %.0f ppb",  
                              mirics.sampling_speed,
                              10.*mirics.freq_adjust);
  lir_text(24,line[0],s);
  SNDLOG"\n%s",s);
  }
return (errcod);
}


void init_mirics(void)
{
FILE *mirics_file;
int i, line, no_of_mirics, devno;
int *ise, *sdr_pi;
char vendor[256], product[256], serial[256];
char s[120];
ise=(int*)(void*)serial;
line=0;
no_of_mirics = mirisdr_get_device_count();
if (no_of_mirics == 0) 
  {
  lir_text(5,5,"Mirics not found.");
  lir_text(5,7,press_any_key);
  await_keyboard();
  return;
  }
line=2;
for(i=0; i<no_of_mirics; i++)
  {
  mirisdr_get_device_usb_strings(i, vendor, product, serial);
  sprintf(s," %2d: %s   MFG:%s, PROD:%s, SN: %s.", i, 
                mirisdr_get_device_name(i), vendor, product, serial);
  lir_text(3,line,s);
  line++;
  }
line++;
lir_text(3, line, "Select device by line number:");
devno=lir_get_integer(32,line,2,0,no_of_mirics-1);
mirisdr_get_device_usb_strings(devno, vendor, product, serial);
mirics.sernum1=ise[0];
mirics.sernum2=ise[1];
mirics.sernum3=ise[2];
line+=2;
lir_sleep(100000);
i=mirisdr_open(&dev_mirics, (uint32_t)devno);
if(i<0)
  {
#if(OSNUM == OSNUM_WINDOWS)
  lirerr(1348);
#else
  lirerr(1357);
#endif
  return;
  }
getspeed:;
sprintf(s,"Set sampling speed in Hz (%d to %d) =>",1,99999999);
lir_text(3,line,s);
mirics.sampling_speed=lir_get_integer(4+strlen(s),line,10,
                                1,99999999);
line+=2;
i=mirisdr_set_sample_rate(dev_mirics, (uint32_t)mirics.sampling_speed);
mirics.sampling_speed=mirisdr_get_sample_rate(dev_mirics);
mirisdr_close(dev_mirics);
if(i < 0)
  {
  lir_text(3,line,"Failed to set sampling rate");
  lir_text(3,line+1,press_any_key);
  await_keyboard();
  clear_lines(line,line+1);
  goto getspeed;
  }
line++;
sprintf(s,"Actual sampling speed is %d Hz",mirics.sampling_speed);
lir_text(3,line,s);
line+=2;
lir_text(3,line,"Enter xtal error in ppb =>");
mirics.freq_adjust=0.1*lir_get_float(32,line,9,-300000.,300000.);
mirics_file=fopen(mirics_parfil_name,"w");
if(mirics_file == NULL)
  {
  lirerr(381268);
  return;
  }
mirics.check=MIRISDRPAR_VERNR;
sdr_pi=(int*)(&mirics);
for(i=0; i<MAX_MIRISDR_PARM; i++)
  {
  fprintf(mirics_file,"%s [%d]\n",mirics_parm_text[i],sdr_pi[i]);
  }
parfile_end(mirics_file);
ui.rx_addev_no=MIRISDR_DEVICE_CODE;
ui.rx_ad_speed=(int)((rint)((mirics.sampling_speed*
                     (100000000L+(double)mirics.freq_adjust))/100000000L));
ui.rx_input_mode=IQ_DATA;
ui.rx_rf_channels=1;
ui.rx_ad_channels=2;
ui.rx_admode=0;

}
#endif

